package luk;

import static org.fest.assertions.Assertions.*;
import static org.junit.Assert.*;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.definition.type.FactType;
import org.drools.io.ResourceFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.WorkItem;
import org.drools.runtime.process.WorkItemHandler;
import org.drools.runtime.process.WorkItemManager;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public class SimpleTest {

    private PoolingDataSource    ds;
    private KnowledgeBase        kbase;
    private Environment          environment;
    private EntityManagerFactory emf;
    private long                 workItemId;

    @Before
    public void setup() throws Exception {
        kbase = kbase();
        prepareDataSource();
        environment = environment();
    }

    @After
    public void tearDown() throws Exception {
        TransactionManagerServices.getTransactionManager().shutdown();
        emf.close();
        ds.close();
    }

    @Test
    public void shouldSerializeVariable() throws Exception {
        FactType type = kbase.getFactType("luk", "Stuff");
        Object object = type.newInstance();
        type.set(object, "status", "A");
        StatefulKnowledgeSession ksession = null;
        ksession = newSession();
        int sessionId = ksession.getId();
        WorkItemManager manager = ksession.getWorkItemManager();
        manager.registerWorkItemHandler("Task", new WorkItemHandler() {

            @Override
            public void executeWorkItem(WorkItem workItem, WorkItemManager manager) {
                workItemId = workItem.getId();
            }

            @Override
            public void abortWorkItem(WorkItem workItem, WorkItemManager manager) {
            }
        });
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("object", object);
        ksession.insert(object);
        Collection<Object> objects = ksession.getObjects();
        assertThat(objects).hasSize(1);
        Object o = objects.iterator().next();
        assertEquals("A", type.get(o, "status"));
        ksession.startProcess("process", parameters);
        ksession.fireAllRules();
        objects = ksession.getObjects();
        assertThat(objects).hasSize(1);
        o = objects.iterator().next();
        assertEquals("X", type.get(o, "status"));
        ksession.dispose();

        ksession = loadSession(sessionId);
        ksession.getWorkItemManager().completeWorkItem(workItemId, null);
        objects = ksession.getObjects();
        assertThat(objects).hasSize(1);
        o = objects.iterator().next();
        assertEquals("Y", type.get(o, "status"));
        ksession.dispose();
    }

    private KnowledgeBase kbase() {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newClassPathResource("Stuff.drl"), ResourceType.DRL);
        kbuilder.add(ResourceFactory.newClassPathResource("process.bpmn"), ResourceType.BPMN2);
        if (kbuilder.hasErrors()) {
            throw new RuntimeException(kbuilder.getErrors().toString());
        }
        return kbuilder.newKnowledgeBase();
    }

    private StatefulKnowledgeSession newSession() {
        return JPAKnowledgeService.newStatefulKnowledgeSession(kbase, null, environment);
    }

    private StatefulKnowledgeSession loadSession(int sessionId) {
        return JPAKnowledgeService.loadStatefulKnowledgeSession(sessionId, kbase, null, environment);
    }

    private Environment environment() {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa");
        Environment env = KnowledgeBaseFactory.newEnvironment();
        env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
        return env;
    }

    private void prepareDataSource() {
        ds = new PoolingDataSource();
        ds.setUniqueName("java:comp/env/jdbc/mydatasource");
        ds.setClassName("org.h2.jdbcx.JdbcDataSource");
        ds.setMaxPoolSize(3);
        ds.setAllowLocalTransactions(true);
        ds.getDriverProperties().put("user", "sa");
        ds.getDriverProperties().put("URL", "jdbc:h2:mem:");
        ds.init();
        Map<String, Object> configOverrides = new HashMap<String, Object>();
        configOverrides.put("hibernate.hbm2ddl.auto", "create");
        emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa", configOverrides);
    }
}
